
function qqnew = datastat_del(qq, nd, ss, dtype, dparam)
%datastat_del    Delete single data item from cached statistics for one class

numdata = size(ss,2);

switch dtype,
case 1,   % Discrete multinomial
  qqnew     = qq;
  for ii = 1:numdata
    qqnew(ss(ii)) = qqnew(ss(ii)) - 1;
  end

case 2,   % Normal-Inverse-Wishart
  dim   = dparam(1);
  nu    = dparam(2) + nd;
  kappa = dparam(3) + nd;

  % extract sum and Cholesky decomposition of outer product
  datasum  = qq(1:dim);
  datachol = reshape(qq(dim+1:end-1),dim,dim);

  % add back outer product of "old" datasum
  datachol = cholupdate(datachol, datasum./sqrt(kappa));

  % remove given data item from cached sum and outer product 
  datasum  = datasum - sum(ss,2);
  for ii = 1:numdata
    datachol = cholupdate(datachol, ss(:,ii), '-');
  end
  kappa = kappa - numdata;
  nu    = nu - numdata;

  % subtract outer product of "new" datasum
  datachol = cholupdate(datachol, datasum./sqrt(kappa), '-');

  % store logdet normalization for faster likelihood evaluation
  covnorm = (kappa+1)/(kappa*(nu-dim-1));
  logdet  = -0.5*dim*log(covnorm) - sum(log(diag(datachol)));
  logdet  = logdet - 0.5*dim*log(2*pi);

  qqnew   = [datasum; datachol(:); logdet];

case 3,   % Diagonal Normal-Inverse-Wishart
  dim   = dparam(1);
  nu    = dparam(2) + nd;
  kappa = dparam(3) + nd;

  % extract sum and Cholesky decomposition of outer product
  datasum  = qq(1:dim);
  datachol = qq(dim+1:end-1);

  % add back outer product of "old" datasum
  datachol = datachol.^2 + (datasum.^2)/kappa;

  % add new data item to cached sum and outer product 
  datasum  = datasum - sum(ss,2);
  datachol = datachol - sum(ss.^2,2);
  kappa    = kappa - numdata;
  nu       = nu - numdata;

  % subtract outer product of "new" datasum
  datachol = sqrt(datachol - (datasum.^2)/kappa);

  % store logdet normalization for faster likelihood evaluation
  covnorm = (kappa+1)/(kappa*(nu-dim-1));
  logdet  = -0.5*dim*log(covnorm) - sum(log(datachol));
  logdet  = logdet - 0.5*dim*log(2*pi);

  qqnew   = [datasum; datachol; logdet];
 
case 4,   % Inverse-Wishart (zero mean)
  dim = dparam(1);
  nu  = dparam(2) + nd;

  % extract Cholesky decomposition of outer product
  datachol = reshape(qq(1:end-1), dim, dim);

  % remove given data item from cached sum and outer product 
  for ii = 1:numdata
    datachol = cholupdate(datachol, ss(:,ii), '-');
  end

  % store logdet normalization for faster likelihood evaluation
  logdet  = 0.5*dim*log(nu-numdata-dim-1) - sum(log(diag(datachol)));
  logdet  = logdet - 0.5*dim*log(2*pi);

  qqnew   = [datachol(:); logdet];

case 5,   % Diagonal Inverse-Wishart (zero mean)
  dim = dparam(1);
  nu  = dparam(2) + nd;

  datachol = qq(1:dim);
  datachol = sqrt(datachol.^2 - sum(ss.^2,2));

  % store logdet normalization for faster likelihood evaluation
  logdet  = 0.5*dim*log(nu-numdata-dim-1) - sum(log(datachol));
  logdet  = logdet - 0.5*dim*log(2*pi);

  qqnew   = [datachol; logdet];
  
end

