
function theta = datastat_estparam(qq, nd, dtype, dparam)
%datastat_estparam     Estimate most likely parameters for multiple classes

switch dtype,
case 1,   % Discrete multinomial
  numfeature = dparam(1);
  dirweight  = dparam(2);

  Z     = nd + dirweight*numfeature;
  theta = qq ./ repmat(Z,numfeature,1);

case 2,   % Normal-Inverse-Wishart
  dim   = dparam(1);
  nu    = dparam(2) + nd;
  kappa = dparam(3) + nd;

  numcc = size(qq,2);
  theta = zeros(dim+dim^2,numcc);

  datasum  = qq(1:dim,:);
  datachol = reshape(qq(dim+1:end-1,:), dim, dim, numcc);
  covnorm  = (kappa+1) ./ (kappa.*(nu-dim-1));
  
  theta(1:dim,:) = datasum ./ repmat(kappa,dim,1);
  for cc = 1:numcc
    sigma = covnorm(cc)*datachol(:,:,cc)'*datachol(:,:,cc);
    theta(dim+1:end,cc) = sigma(:);
  end
  
case 3,   % Diagonal Normal-Inverse-Wishart
  dim   = dparam(1);
  nu    = dparam(2) + nd;
  kappa = dparam(3) + nd;

  numcc = size(qq,2);
  theta = zeros(2*dim,numcc);

  datasum  = qq(1:dim,:);
  datachol = qq(dim+1:end-1,:);
  covnorm  = (kappa+1) ./ (kappa.*(nu-dim-1));
  
  theta(1:dim,:)     = datasum ./ repmat(kappa,dim,1);
  theta(dim+1:end,:) = (datachol.^2) .* repmat(covnorm,dim,1);

case 4,   % Inverse-Wishart (zero mean)
  dim = dparam(1);
  nu  = dparam(2) + nd;

  numcc = size(qq,2);
  theta = zeros(dim^2,numcc);

  datachol = reshape(qq(1:end-1,:), dim, dim, numcc);
  covnorm  = 1 ./ (nu-dim-1);
  
  for cc = 1:numcc
    sigma = covnorm(cc)*datachol(:,:,cc)'*datachol(:,:,cc);
    theta(:,cc) = sigma(:);
  end

case 5,   % Diagonal Inverse-Wishart (zero mean)
  dim = dparam(1);
  nu  = dparam(2) + nd;

  numcc = size(qq,2);

  if numcc > 0
    datachol = qq(1:dim,:);
    covnorm  = 1 ./ (nu-dim-1);

    theta = (datachol.^2) .* covnorm(ones(dim,1),:);
  else
    theta = zeros(dim,0);
  end

end

