
function clik = datastat_indeplike(qq, nd, ss, dtype, dparam)
%datastat_indeplike    Likelihoods of several data points, under multiple classes

numdata = size(ss,2);

switch dtype,
case 1,   % Discrete multinomial
  numfeature = dparam(1);
  dirweight  = dparam(2);

  % vectorized computation of likelihoods of all data points
  logZ = log(nd + dirweight*numfeature);
  clik = log(qq(ss',:)) - logZ(ones(numdata,1),:);

case 2,   % Normal-Inverse-Wishart
  dim   = dparam(1);
  nu    = dparam(2) + nd;
  kappa = dparam(3) + nd;
  numcc = size(qq,2);

  datasum  = qq(1:dim,:);
  datachol = reshape(qq(dim+1:end-1,:), dim, dim, numcc);

  % vectorized computation of likelihoods of all data points
  clik      = qq(end,:);
  clik      = clik(ones(numdata,1),:);
  innernorm = 0.5*(nu-dim-1) ./ (kappa.*(kappa+1));
  
  for cc = 1:numcc
    delmu = datachol(:,:,cc)' \ (datasum(:,cc*ones(1,numdata))-kappa(cc)*ss);
    clik(:,cc) = clik(:,cc) - innernorm(cc)*sum(delmu.^2,1)';
  end

case 3,   % Diagonal Normal-Inverse-Wishart
  dim   = dparam(1);
  nu    = dparam(2) + nd;
  kappa = dparam(3) + nd;
  numcc = size(qq,2);

  datasum  = qq(1:dim,:);
  datachol = qq(dim+1:end-1,:);

  % vectorized computation of likelihoods of all data points
  clik      = qq(end,:);
  clik      = clik(ones(numdata,1),:);
  innernorm = 0.5*(nu-dim-1) ./ (kappa.*(kappa+1));

  for cc = 1:numcc
    delmu = datasum(:,cc*ones(1,numdata)) - kappa(cc)*ss;
    delmu = delmu ./ datachol(:,cc*ones(1,numdata));
    clik(:,cc) = clik(:,cc) - innernorm(cc)*sum(delmu.^2,1)';
  end

case 4,   % Inverse-Wishart (zero mean)
  dim   = dparam(1);
  nu    = dparam(2) + nd;
  numcc = size(qq,2);

  datachol = reshape(qq(1:end-1,:), dim, dim, numcc);

  % vectorized computation of likelihoods of all data points
  clik      = qq(end,:);
  clik      = clik(ones(numdata,1),:);
  innernorm = 0.5*(nu-dim-1);

  for cc = 1:numcc
    delmu = datachol(:,:,cc)' \ ss;
    clik(:,cc) = clik(:,cc) - innernorm(cc)*sum(delmu.^2,1)';
  end

case 5,   % Diagonal Inverse-Wishart (zero mean)
  dim   = dparam(1);
  nu    = dparam(2) + nd;
  numcc = size(qq,2);

  datachol = qq(1:end-1,:);

  % vectorized computation of likelihoods of all data points
  clik      = qq(end,:);
  clik      = clik(ones(numdata,1),:);
  innernorm = 0.5*(nu-dim-1);

  for cc = 1:numcc
    delmu = ss ./ datachol(:,cc*ones(1,numdata));
    clik(:,cc) = clik(:,cc) - innernorm(cc)*sum(delmu.^2,1)';
  end

end

