
function theta = datastat_sampparam(qq, nd, numsamp, dtype, dparam)
%datastat_sampparam     Sample from posterior parameter density for one class

switch dtype,
case 1,   % Discrete multinomial
  theta = dirichlet_rnd(qq, numsamp);

case 2,   % Normal-Inverse-Wishart
  dim   = dparam(1);
  nu    = dparam(2) + nd;
  kappa = dparam(3) + nd;

  datasum  = qq(1:dim);
  datachol = reshape(qq(dim+1:end-1),dim,dim);

  mu    = datasum ./ kappa;
  scale = datachol'*datachol;

  theta  = zeros(dim+dim^2,numsamp);
  for pp = 1:numsamp
    sigmaP = invwishart_rnd(scale, nu, 1);
    muP    = mu + chol(sigmaP./kappa)'*randn(dim,1);

    theta(:,pp) = [muP; sigmaP(:)];
  end
  
case 3,   % Diagonal Normal-Inverse-Wishart
  dim   = dparam(1);
  nu    = dparam(2) + nd;
  kappa = dparam(3) + nd;

  mu        = qq(1:dim) ./ kappa;
  sqrtscale = qq(dim+1:end-1);

  theta  = zeros(2*dim,numsamp);
  for pp = 1:numsamp
    covsamp = randn(dim,nu) ./ repmat(sqrtscale,1,nu);
    sigmaP  = 1 ./ sum(covsamp.^2,2);
    muP    = mu + sqrt(sigmaP./kappa) .* randn(dim,1);

    theta(:,pp) = [muP; sigmaP];
  end

case 4,   % Inverse-Wishart (zero mean)
  dim = dparam(1);
  nu  = dparam(2) + nd;

  datachol = reshape(qq(1:end-1), dim, dim);
  scale    = datachol'*datachol;

  theta  = zeros(dim^2,numsamp);
  for ii = 1:numsamp
    sigmaP = invwishart_rnd(scale, nu, 1);
    theta(:,ii) = sigmaP(:);
  end

case 5,   % Diagonal Inverse-Wishart (zero mean)
  dim = dparam(1);
  nu  = dparam(2) + nd;

  datachol = qq(1:dim);
  scale    = datachol.^2;

  theta = zeros(dim,numsamp);
  for dd = 1:dim
    theta(dd,:) = invwishart_rnd(scale(dd), nu, numsamp);
  end

end

