
function llik = trandata_indeplike(qq, nd, pp, nt, ss, dtype, dparam, ttype, tparam)
%trandata_indeplike    Marginal log-likelihood of several independent observations

switch ttype,
case 1,   % Inverse-Wishart (constrained zero mean, assumes dtype=2)
  dim    = tparam(1); % = dparam(1);
  tranNu = tparam(2) + nt;
  dataNu = dparam(2) + nd;
  kappa  = dparam(3) + nd;

  numcc  = size(qq,2);
  numobs = size(ss,2);
  
  % cached statistics of transformations and data, for each class
  tranchol = reshape(pp(1:end-1,:), dim, dim, numcc);
  datasum  = qq(1:dim,:);
  datachol = reshape(qq(dim+1:end-1,:), dim, dim, numcc);

  % inverse normalization constants for covariance of each class
  trannorm = (tranNu-dim-1);
  datanorm = (kappa.*(dataNu-dim-1)) ./ (kappa+1);

  llik = zeros(numobs,numcc);
  for cc = 1:numcc
    datamean = datasum(:,cc) ./ kappa(cc);

    % compute covariance averaging over transformations
    trancov = tranchol(:,:,cc)'*tranchol(:,:,cc) ./ trannorm(cc);
    datacov = datachol(:,:,cc)'*datachol(:,:,cc) ./ datanorm(cc);
    sumchol = chol(trancov + datacov);

    delmu = sumchol' \ (ss - datamean(:,ones(1,numobs)));
    llik(:,cc) = -sum(log(diag(sumchol))) - 0.5*sum(delmu.^2,1)';
  end
  llik = llik - 0.5*dim*log(2*pi);

case 2,   % Diagonal Inverse-Wishart (constrained zero mean, assumes dtype=3)
  dim    = tparam(1); % = dparam(1);
  tranNu = tparam(2) + nt;
  dataNu = dparam(2) + nd;
  kappa  = dparam(3) + nd;

  numcc  = size(qq,2);
  numobs = size(ss,2);
  
  % cached statistics of transformations and data, for each class
  tranchol = reshape(pp(1:end-1,:), dim, dim, numcc);
  datasum  = qq(1:dim,:);
  datachol = reshape(qq(dim+1:end-1,:), dim, dim, numcc);

  % inverse normalization constants for covariance of each class
  trannorm = (tranNu-dim-1);
  datanorm = (kappa.*(dataNu-dim-1)) ./ (kappa+1);

  % compute covariance averaging over transformations
  trancov  = (tranchol.^2) ./ trannorm(ones(dim,1),:);
  datacov  = (datachol.^2) ./ datanorm(ones(dim,1),:);
  sumchol  = sqrt(trancov + datacov);

  llik = zeros(numobs,numcc);
  for cc = 1:numcc
    datamean = datasum(:,cc) ./ kappa(cc);
    delmu    = (ss - datamean(:,ones(1,numobs))) ./ sumchol(:,cc*ones(1,numobs));
    llik(:,cc) = -sum(log(sumchol(:,cc))) - 0.5*sum(delmu.^2,1)';
  end
  llik = llik - 0.5*dim*log(2*pi);

end

