
function llik = trandata_marglike(qq, nd, pp, nt, ss, dtype, dparam, ttype, tparam)
%trandata_marglike    Marginal log-likelihood of data, integrating over transforms

switch ttype,
case 1,   % Inverse-Wishart (constrained zero mean, assumes dtype=2)
  dim    = tparam(1); % = dparam(1);
  tranNu = tparam(2) + nt;
  dataNu = dparam(2) + nd;
  kappa  = dparam(3) + nd;

  numcc  = size(qq,2);
  numobs = size(ss,2);

  % cached statistics of transformations and data, for each class
  tranchol = reshape(pp(1:end-1,:), dim, dim, numcc);
  datasum  = qq(1:dim,:);
  datachol = reshape(qq(dim+1:end-1,:), dim, dim, numcc);

  % inverse normalization constants for covariance of each class
  trannorm = (tranNu-dim-1);
  datanorm = (kappa.*(dataNu-dim-1)) ./ (kappa+1);

 if numobs == 1
  llik = zeros(1,numcc);
  for cc = 1:numcc
    datamean = datasum(:,cc) ./ kappa(cc);

    % compute covariance averaging over transformations
    trancov = tranchol(:,:,cc)'*tranchol(:,:,cc) ./ trannorm(cc);
    datacov = datachol(:,:,cc)'*datachol(:,:,cc) ./ datanorm(cc);
    sumchol = chol(trancov + datacov);

    delmu    = sumchol' \ (ss-datamean);
    llik(cc) = -sum(log(diag(sumchol))) - 0.5*sum(delmu.^2);
  end
  llik = llik - 0.5*dim*log(2*pi);
 else
  sumobs = sum(ss,2);
  
  % initialize likelihoods with cached log-det-covariances
  llik = qq(end,:)*numobs + pp(end,:);

  for cc = 1:numcc
    datamean = datasum(:,cc) ./ kappa(cc);

    % compute posterior inverse covariance
    trancholinv = tranchol(:,:,cc) \ eye(dim);
    traninv     = trannorm(cc)*trancholinv*trancholinv';
    datacholinv = datachol(:,:,cc) \ eye(dim);
    datainv     = datanorm(cc)*datacholinv*datacholinv';
    postinv     = traninv + numobs*datainv;
    postinvchol = chol(postinv);

    % compute posterior mean
    postmean = postinv \ (datainv * (sumobs-numobs*datamean));

    % compute log-likelihood
    llik(cc) = llik(cc) - sum(log(diag(postinvchol)));
    llik(cc) = llik(cc) + 0.5*sum((postinvchol*postmean).^2);
    
    delmu    = datacholinv'*(ss - datamean(:,ones(1,numobs)));
    llik(cc) = llik(cc) - 0.5*datanorm(cc)*sum(sum(delmu.^2));
  end
 end

case 2,   % Diagonal Inverse-Wishart (constrained zero mean, assumes dtype=3)
  dim    = tparam(1); % = dparam(1);
  tranNu = tparam(2) + nt;
  dataNu = dparam(2) + nd;
  kappa  = dparam(3) + nd;

  numcc  = size(qq,2);
  numobs = size(ss,2);

  % cached statistics of transformations and data, for each class
  tranchol = pp(1:dim,:);
  datamean = qq(1:dim,:) ./ kappa(ones(dim,1),:);
  datachol = qq(dim+1:end-1,:);

  % inverse normalization constants for covariance of each class
  trannorm = (tranNu-dim-1);
  datanorm = (kappa.*(dataNu-dim-1)) ./ (kappa+1);

 if numobs == 1
  trancov  = (tranchol.^2) ./ trannorm(ones(dim,1),:);
  datacov  = (datachol.^2) ./ datanorm(ones(dim,1),:);
  sumchol  = sqrt(trancov + datacov);

  delmu = (ss(:,ones(1,numcc))-datamean) ./ sumchol;
  llik  = -sum(log(sumchol)) - 0.5*sum(delmu.^2);
  llik  = llik - 0.5*dim*log(2*pi);
 else
  sumobs = sum(ss,2);
  
  % initialize likelihoods with cached log-det-covariances
  llik = qq(end,:)*numobs + pp(end,:);

  % compute posterior inverse covariance
  traninv = trannorm(ones(dim,1),:) ./ (tranchol.^2);
  datainv = datanorm(ones(dim,1),:) ./ (datachol.^2);
  postinv = traninv + numobs*datainv;

  % compute posterior mean
  postmean = (datainv .* (sumobs(:,ones(1,numcc))-numobs*datamean)) ./ postinv;

  % compute log-likelihood
  llik = llik - 0.5*sum(log(postinv)) + 0.5*sum(postinv .* (postmean.^2));

  for cc = 1:numcc
    delmu    = ss - datamean(:,cc*ones(1,numobs));
    llik(cc) = llik(cc) - 0.5*sum(datainv(:,cc).*sum(delmu.^2,2));
  end
 end

end

