
function ppnew = transtat_del(pp, nd, rr, ttype, tparam)
%transtat_del    Delete single transformation from cached statistics for one class

switch ttype,
case 0,   % No transformations
  ppnew = 0;

case 1,   % Inverse-Wishart (constrained zero mean)
  dim = tparam(1);
  nu  = tparam(2) + nd;

  tranchol = reshape(pp(1:end-1), dim, dim);
  tranchol = cholupdate(tranchol, rr, '-');

  % store logdet normalization for faster likelihood evaluation
  logdet  = 0.5*dim*log(nu-1-dim-1) - sum(log(diag(tranchol)));
  logdet  = logdet - 0.5*dim*log(2*pi);

  ppnew   = [tranchol(:); logdet];

case 2,   % Diagonal Inverse-Wishart (constrained zero mean)
  dim = tparam(1);
  nu  = tparam(2) + nd;

  tranchol = pp(1:dim);
  tranchol = sqrt(tranchol.^2 - rr.^2);

  % store logdet normalization for faster likelihood evaluation
  logdet  = 0.5*dim*log(nu-1-dim-1) - sum(log(tranchol));
  logdet  = logdet - 0.5*dim*log(2*pi);

  ppnew   = [tranchol; logdet];

end

