
function [pp0,ttype,tparam] = transtat_init(tranprior)
%transtat_init    Initialize cached statistics using given hyperparameters

switch lower(tranprior.type)
case 'zeromeangaussian',   % Inverse-Wishart (constrained zero mean)
  ttype = 1;

  % hyperparameters
  nu0    = tranprior.ccovar;
  sigma0 = tranprior.ecovar;
  dim    = size(sigma0,1);

  % initialize transformation statistics
  tranchol = chol(nu0*sigma0);

  % store logdet normalization for faster likelihood evaluation
  logdet  = 0.5*dim*log(nu0-dim-1) - sum(log(diag(tranchol)));
  logdet  = logdet - 0.5*dim*log(2*pi);

  pp0    = [tranchol(:); logdet];
  tparam = [dim; nu0];

case 'zeromeandiagonalgaussian',   % Diagonal Inverse-Wishart (constrained zero mean)
  ttype = 2;

  % hyperparameters
  nu0    = tranprior.ccovar;
  sigma0 = tranprior.ecovar;

  % handle expected covariance in matrix or vector form
  if prod(size(sigma0)) > length(sigma0)
    sigma0 = diag(sigma0);
  end
  dim = length(sigma0);

  % initialize transformation statistics
  tranchol = sqrt(nu0*sigma0);

  % store logdet normalization for faster likelihood evaluation
  logdet  = 0.5*dim*log(nu0-dim-1) - sum(log(tranchol));
  logdet  = logdet - 0.5*dim*log(2*pi);

  pp0    = [tranchol; logdet];
  tparam = [dim; nu0];

otherwise,
  warning('Unknown transformation type, ignoring');
  ttype  = 0;
  pp0    = 0;
  tparam = 0;

end

