
function catprob = estcategory(objpart, groups)
%estcategory    Posterior category log-likelihoods for current assignments
%
%    catprob = estcategory(OP, groups)
%
%  Compute posterior object category log-likelihoods implied by current
%  assignments of data to classes, where
%    groups  = groups for which probabilites computed (all, by default)
%    catprob = (numobj x length(groups)) matrix of these probabilities

  % default parameters
  if nargin < 2
    groups = [1:objpart.numgroup];
  end

  % static parameters
  alpha    = objpart.alpha;
  numobj   = objpart.numobj;
  numclass = objpart.numclass;
  numtype  = objpart.numtype;
  dtype    = objpart.dtype;
  dparam   = objpart.dparam;

  % class (part) level state variables
  classnd = objpart.classnd;
  classqq = objpart.classqq;

  % compute P(part|object)
  partprob = objpart.objnd + alpha;
  probnorm = sum(partprob,2);
  partprob = partprob ./ repmat(probnorm,1,numclass);

  % initialize output data structure
  numgroup = size(groups,2);
  catprob  = zeros(numobj,numgroup);

  for j = 1:numgroup
    jj = groups(j);

    % data for current group
    numdata = objpart.numdata(jj);
    datass  = objpart.datass{jj};

    % compute logP(feature|part)
    clik = zeros(numdata,numclass);
    for dd = 1:numtype
      classqqLocal = classqq{dd};
      datassLocal  = datass{dd};
      dtypeLocal   = dtype(dd);
      dparamLocal  = dparam{dd};

      for ii = 1:numdata
        clik(ii,:) = clik(ii,:) + datastat_condlike(classqqLocal, ...
          classnd, datassLocal(:,ii), dtypeLocal, dparamLocal);
      end
    end

    % compute P(feature|object)
    featprob = exp(clik) * partprob';

    % compute unnormalized log-probability to allow comparison to other models
    catprob(:,j) = sum(log(featprob),1)'; 
    %catprob(:,j) = exp(catprob(:,j) - max(catprob(:,j)));
    %catprob(:,j) = catprob(:,j) / sum(catprob(:,j));
  end

