
function objpart = insertdata(objpart, newdata, group, category)
%insertdata     Add new data items to the specified group
%
%    OP = insertdata(OP, newdata, group, category)
%
%  Adds new data items to specified group, where
%    newdata  = matrix such that each column is a different data point,
%               or if multiple datatypes a cell array such that
%               newdata{dd} is the data matrix for datatype dd
%    group    = single group in which to add data (existing data
%               in that group is appended to, not replaced)
%    category = object category label for this group, or 0 if a test
%               group with unspecified category (default)

  if nargin < 3
    group = 1;
  end
  if nargin < 4
    category = 0;
  end

  % Check that newdata has proper number of types and is consistent
  if ~iscell(newdata)
    newdata = {newdata};
  end
  if length(newdata) ~= objpart.numtype
    error('Wrong number of types in newdata');
  end
  for dd = 2:objpart.numtype
    if size(newdata{dd},2) ~= size(newdata{1},2)
      error('Internal inconsistency in newdata');
    end
  end

  % Check consistency of category label
  if (objpart.category(group) > 0) && (category ~= objpart.category(group))
    error('Supplied category label does not match this group');
  end
  objpart.category(group) = category;

  % Add data to specified group
  objpart.numdata(group)  = objpart.numdata(group) + size(newdata{1},2);
  objpart.datacc{group}   = [objpart.datacc{group}, zeros(1, size(newdata{1},2))];
  for dd = 1:objpart.numtype
    objpart.datass{group}{dd}   = [objpart.datass{group}{dd}, newdata{dd}];
  end

