
function [classParam,classProb] = getclasses(sceneobj)
%getclasses     Estimate posterior class parameters from current assignments
%
%    [classParam,classProb] = getclasses(SO)
%
%  Estimate class parameters corresponding to current data assignments, where
%    classParam = cell array such that each column of classParam{dd} gives 
%                 the parameters for a different class, for datatype dd
%    classProb  = (numobj x numclass) matrix giving P(class|category)

% Hyperparameters
numclass = sceneobj.numclass;
numtype  = sceneobj.numtype;
alpha    = sceneobj.alpha;
classqq  = sceneobj.classqq;
classnd  = sceneobj.classnd;
dtype    = sceneobj.dtype;
dparam   = sceneobj.dparam;

% Posterior part parameters implied by current assignments
classParam = cell(numtype,1);
for dd = 1:numtype
  classParam{dd} = datastat_estparam(classqq{dd}, classnd, dtype(dd), dparam{dd});
end

% Category-specific part probabilities implied by current assignments
if nargout > 1
  classProb = sceneobj.objnd + alpha;
  Z = sum(classProb,2);
  classProb = classProb ./ repmat(Z,1,numclass);
end

