
function [sceneProb,refMean,refCovar] = getscenes(sceneobj)
%getscenes     Estimate posterior scene parameters from current assignments
%
%    [sceneProb,refMean,refCovar] = getscenes(SO)
%
%  Estimate scene parameters corresponding to current data assignments, where
%    sceneProb = (numobj x numscene) matrix giving P(object|scene)
%    refMean   = matrix such that refMean(:,s) is a dim*numobj vector
%                giving mean reference positions for scene s
%    refCovar  = matrix such that refCovar(:,:,s) is the reference position
%                covariance matrix for scene s

% Hyperparameters
numscene = sceneobj.numscene;
numobj   = sceneobj.numobj;
gamma    = sceneobj.gamma;

% Reference transformation state variables
refsum   = sceneobj.refsum;
refchol  = sceneobj.refchol;

% Scene-specific object probabilities implied by current assignments
sceneProb = sceneobj.scenend + gamma;
Z = sum(sceneProb,1);
sceneProb = sceneProb ./ Z(ones(numobj,1),:);

% Posterior reference distributions implied by current reference samples
if sceneobj.useref
  dim      = sceneobj.rparam(1);
  refnu    = sceneobj.rparam(2) + sceneobj.sceneng;
  refkappa = sceneobj.rparam(3) + sceneobj.sceneng;
  covnorm  = (refkappa+1)./(refkappa.*(refnu-numobj*dim-1));

  refMean  = refsum ./ refkappa(ones(dim*numobj,1),:);

  refCovar = zeros(dim*numobj,dim*numobj,numscene);
  for aa = 1:numscene
    refCovar(:,:,aa) = cholupdate(refchol(:,:,aa), ...
      refsum(:,aa) / sqrt(refkappa(aa)), '-');
    refCovar(:,:,aa) = refCovar(:,:,aa)'*refCovar(:,:,aa) * covnorm(aa);
  end
else
  refMean  = zeros(1,numscene);
  refCovar = zeros(1,1,numscene);
end

