
function sceneobj = insertdata(sceneobj, newdata, group, scene, objlabels)
%insertdata     Add new data items to the specified group
%
%    SO = insertdata(SO, newdata, group, scene, objlabels)
%
%  Adds new data items to specified group, where
%    newdata   = matrix such that each column is a different data point,
%                or if multiple datatypes a cell array such that
%                newdata{dd} is the data matrix for datatype dd
%    group     = single group in which to add data (existing data
%                in that group is appended to, not replaced)
%    scene     = scene type label for this group
%    objlabels = vector giving object category label for each data item,
%                or scalar giving category of all data items
%                (set category to 0 to specify unlabeled, the default,
%                 or negative number to exclude assignment to category)

  if nargin < 3
    group = 1;
  end
  if nargin < 4
    scene = 1;
  end
  if nargin < 5
    objlabels = 0;
  end

  % Check that newdata has proper number of types and is consistent
  if ~iscell(newdata)
    newdata = {newdata};
  end
  if length(newdata) ~= sceneobj.numtype
    error('Wrong number of types in newdata');
  end

  numdata = size(newdata{1},2);
  for dd = 2:sceneobj.numtype
    if size(newdata{dd},2) ~= numdata
      error('Internal inconsistency in newdata');
    end
  end

  % Check consistency of scene category
  if (sceneobj.category(group) > 0) && (scene ~= sceneobj.category(group))
    error('Supplied scene category does not match this group');
  end
  if sceneobj.category(group) == 0
    sceneobj.sceneng(scene) = sceneobj.sceneng(scene) + 1;
  end
  sceneobj.category(group) = scene;

  % Check consistency of object labels
  if length(objlabels) == 1
    objlabels = objlabels*ones(1,numdata);
  elseif length(objlabels) ~= numdata
    error('Improper number of object category labels');
  else
    objlabels = reshape(objlabels,1,numdata);
  end
  
  % Add data to specified group
  sceneobj.numdata(group) = sceneobj.numdata(group) + numdata;
  sceneobj.datacc{group}  = [sceneobj.datacc{group}, zeros(1,numdata)];
  sceneobj.dataoo{group}  = [sceneobj.dataoo{group}, objlabels];
  sceneobj.labeloo{group} = [sceneobj.labeloo{group}, objlabels];
  for dd = 1:sceneobj.numtype
    sceneobj.datass{group}{dd} = [sceneobj.datass{group}{dd}, newdata{dd}];
  end

