
function sceneobj = setassign(sceneobj, group, labels)
%setassign    Set labels restricting object category assignments for some data
%
%    SO = setassign(SO, group, labels)
%
%  Sets labels restricting assignment of some features categories, where
%    group  = single group for which assignments set 
%    labels = vector giving object category label for each data item,
%             or scalar giving category of all data items
%             (set category to 0 to specify unlabeled, the default,
%             or negative number to exclude assignment to category)

  % default parameters
  if nargin < 2
    group = 1;
  end
  if nargin < 3
    labels = 0;
  end

  % check consistency of labels with amount of data
  numdata = sceneobj.numdata(group);
  if length(labels) == 1
    labels = labels*ones(1,numdata);
  elseif length(labels) ~= numdata
    error('Improper number of labels specified');
  else
    labels = reshape(labels,1,numdata);
  end

  % set specified labels on data
  % will propagate to assignments on next Gibbs iteration
  sceneobj.labeloo{group} = labels;

