
%test_objectpart    Synthetic data test of object-part model

%%%%%%%%%%%%%
%% Parameters
%%%%%%%%%%%%%

% Clustering hyperparameters
numobj  = 3;
numpart = 5;
alpha   = 1;

useApp = 1; % appearance features: 0=none, 1=discrete, 2=Gaussian (diagonal)
usePos = 1; % 1 to use Gaussian feature position model, 0 neglects positions

% Gaussian position hyperparameters and visualization
dataprior{1}.type    = 'gaussian';
dataprior{1}.emean   = zeros(2,1);
dataprior{1}.cmean   = 0.1;
dataprior{1}.ecovar  = eye(2);
dataprior{1}.ccovar  = 6;
plotstyle{1}.numbers = 0;

% Discrete appearance hyperparameters and visualization
dataprior{2}.type = 'discrete';
dataprior{2}.numfeature = 30;
dataprior{2}.dirweight  = 0.1;
plotstyle{2} = struct([]);

% Diagonal Gaussian appearance hyperparameters and visualization
appdim = 128;  % dimension of diagonal Gaussian appearance features
dataprior{3}.type   = 'diagonalgaussian';
dataprior{3}.emean  = zeros(appdim,1);
dataprior{3}.cmean  = 0.1;
dataprior{3}.ecovar = ones(appdim,1);
dataprior{3}.ccovar = appdim+5;  % must be larger than appdim+1
plotstyle{3}.meanscale  = [-10 10];
plotstyle{3}.stdevscale = [0.5 2];

% Test corpus parameters
numtrain = 50;
numtest  = 50;
numdata  = 100;

% Learning parameters
numgibbs = 100;

%%%%%%%%%%%%%%
%% Test script
%%%%%%%%%%%%%%

% Select appropriate data hyperparameters
if useApp==1
  dataprior = dataprior(1:2);
  plotstyle = plotstyle(1:2);
elseif useApp==2
  dataprior = dataprior([1 3]);
  plotstyle = plotstyle([1 3]);
else
  dataprior = dataprior(1);
  plotstyle = plotstyle(1);
end
if ~usePos
  dataprior = dataprior(2);
  plotstyle = plotstyle(2);
end

% Create figures to display inference results
for dd = 1:length(plotstyle)
  Htgt(dd) = figure('name', 'Ground truth');
  Hinf(dd) = figure('name', 'Inferred parts');
end

% Construct and visualize synthetic data set
OPtgt = objectpart(numtrain+numtest, numobj, numpart, alpha, dataprior);
OPtgt = sampledata(OPtgt, [], 0, numdata);
[tgtParam,tgtProb] = getclasses(OPtgt);

visualize(OPtgt, plotstyle, Htgt);
drawnow;

% Clear part labels used to generate data
OPinf = clearlabels(OPtgt);

% Remove category labels to create test images
if numtest > 0
  OPinf = clearlabels(OPinf,[numtrain+1:numtrain+numtest],1);
end

% Do Gibbs sampling on training, and estimate test probabilities
[OPinf,testprob] = doGibbs(OPinf, numgibbs, {plotstyle, Hinf}, ...
  [1:numtrain], [numtrain+1:numtrain+numtest]);
[infParam,infProb] = getclasses(OPinf);

