
function [classParam,classProb,classTran] = getclasses(tdp)
%getclasses     Estimate posterior class parameters from current assignments
%
%    [classParam,classProb,classTran] = getclasses(TDP)
%
%  Estimate class parameters corresponding to current data assignments, where
%    classParam = cell array such that each column of classParam{dd} gives 
%                 the parameters for a different class, for datatype dd
%    classProb  = numclass-dim. vector giving P(class)
%    classTran  = columns are transformation parameters for each class

% data and transformation hyperparameters
numtype  = tdp.numtype;
dtype    = tdp.dtype;
dparam   = tdp.dparam;
ttype    = tdp.ttype;
tparam   = tdp.tparam;

% class state variables
numclass = tdp.numclass;
classqq  = tdp.classqq;
classpp  = tdp.classpp;
classnd  = tdp.classnd;
classnt  = tdp.classnt;

% class-specific data distributions
classParam = cell(numtype,1);
for dd = 1:numtype
  classParam{dd} = datastat_estparam(classqq{dd}, classnd, ...
    dtype(dd), dparam{dd});
end

% probability of each class
classProb = classnt/sum(classnt);

% distributions over class-specific transformations
classTran = transtat_estparam(classpp, classnt, ttype, tparam);

