
function [tableClass,tableProb,tableTran,tableAssign] = gettables(tdp,groups)
%gettables    Transformed clusters, or tables, for specified groups
%
%    [tableClass,tableProb,tableTran,tableAssign] = gettables(TDP, groups)
%
%  Estimate transformed clusters, or tables, from current assignments, 
%  where all outputs are length(groups)-dim. cell arrays such that
%    tableClass{j} = numtable-dim. vector giving global class for each table
%    tableProb{j}  = numtable-dim. vector giving P(table)
%    tableTran{j}  = columns are transformation parameters for each table
%    tableAssign{j}= index of table to which each data item in group assigned

if nargin < 2
  groups = [1:tdp.numgroup];
end

% data structures to store results
tableClass  = cell(length(groups),1);
tableProb   = cell(length(groups),1);
tableTran   = cell(length(groups),1);
tableAssign = cell(length(groups),1);

% compute transformed mixtures, or tables, for each specified group
for j=1:length(groups)
  jj = groups(j);

  datatt   = tdp.datatt{jj};
  tablecc  = tdp.tablecc{jj};
  tablerr  = tdp.tablerr{jj};
  tablend  = tdp.tablend{jj};

  tableClass{j}  = tablecc;
  tableProb{j}   = tablend/sum(tablend);
  tableTran{j}   = tablerr;
  tableAssign{j} = datatt;
end

