
function alpha = gibbs_conparam(alpha, numdata, numclass, aa, bb, numiter)
%gibbs_conparam     Auxiliary variable resampling of DP concentration parameter

if nargin < 6
  numiter = 1;
end

numgroup   = length(numdata);
totalclass = sum(numclass);

xx = zeros(1,numgroup);

for ii = 1:numiter
  % beta auxiliary variables
  for jj = 1:numgroup
    xj     = dirichlet_rnd([alpha+1 numdata(jj)], 1);
    xx(jj) = xj(1);
  end

  % binomial auxiliary variables
  zz = (rand(numgroup,1).*(alpha+numdata)) < numdata;

  % gamma resampling of concentration parameter
  gammaa = aa + totalclass - sum(zz);
  gammab = bb - sum(log(xx));
  alpha  = gamm_rnd(1, 1, gammaa, gammab);
end

