
function clik = depth_condlike(qq, nd, rr, ss, dtype, dparam, ztype, zparam)
%depth_condlike    Evaluate likelihoods for multiple classes and depths

%assumes size(ss,2)==1

numzz = zparam(1);
zz    = zparam(2:numzz+1);
numcc = size(qq,2);

% parameterize ray in terms of depth in scene
v = [ss(1:2); 1];

switch dtype,
case 2,   % Inverse-Wishart 
  dim   = dparam(1);
  nu    = dparam(2) + nd;
  kappa = dparam(3) + nd;

  datamean = qq(1:dim,:) ./ kappa(ones(dim,1),:);
  datachol = reshape(qq(dim+1:end-1,:), dim, dim, numcc);
  datanorm = (kappa+1) ./ (kappa.*(nu-dim-1));

  % compute variance, mean, weight of each cluster, conditioned on depth ray
  lamZ = zeros(1,numcc);
  muZ  = zeros(1,numcc);
  logW = qq(end,:);
  for cc = 1:numcc
    scaledV  = datachol(:,:,cc)' \ v;
    scaledMu = datachol(:,:,cc)' \ (datamean(:,cc) + rr(:,cc));

    lamZ(cc) = datanorm(cc) / sum(scaledV.^2);
    muZ(cc)  = (lamZ(cc)/datanorm(cc)) * scaledV'*scaledMu;
    
    delmu    = muZ(cc)*scaledV - scaledMu;
    logW(cc) = logW(cc) - 0.5*sum(delmu.^2)/datanorm(cc);
  end

  % evaluate conditioned Gaussians at discrete set of depths
  % 0.5*log(lamZ) term cancels in calculation of mixture weights
  clik = logW(ones(numzz,1),:);
  clik = clik - 0.5*((zz(:,ones(1,numcc))-muZ(ones(numzz,1),:)).^2 ./ ...
    lamZ(ones(numzz,1),:));

case 3,   % Diagonal Inverse-Wishart 
  dim   = dparam(1);
  nu    = dparam(2) + nd;
  kappa = dparam(3) + nd;

  datamean = qq(1:dim,:) ./ kappa(ones(dim,1),:);
  datachol = qq(dim+1:end-1,:);
  datanorm = (kappa+1) ./ (kappa.*(nu-dim-1));

  % compute variance, mean, weight of each cluster, conditioned on depth ray
  scaledV  = v(:,ones(1,numcc)) ./ datachol;
  scaledMu = (datamean+rr) ./ datachol;

  lamZ  = datanorm ./ sum(scaledV.^2);
  muZ   = (lamZ./datanorm) .* sum(scaledV.*scaledMu);
  delmu = muZ(ones(dim,1),:).*scaledV - scaledMu;
  logW  = qq(end,:) - 0.5*sum(delmu.^2)./datanorm;

  % evaluate conditioned Gaussians at discrete set of depths
  % 0.5*log(lamZ) term cancels in calculation of mixture weights
  clik = logW(ones(numzz,1),:);
  clik = clik - 0.5*((zz(:,ones(1,numcc))-muZ(ones(numzz,1),:)).^2 ./ ...
    lamZ(ones(numzz,1),:));
end

