
function clik = depth_marglike(qq,nd,pp,nt,ss,dtype,dparam,ttype,tparam,ztype,zparam)
%depth_condlike    Evaluate likelihoods for multiple classes and depths

%assumes size(ss,2)==1

numzz = zparam(1);
zz    = zparam(2:numzz+1);
numcc = size(qq,2);

% parameterize ray in terms of depth in scene
v = [ss(1:2); 1];

switch ttype,
case 1,   % Inverse-Wishart (constrained zero mean, assumes dtype=2)
  dim    = tparam(1); % = dparam(1);
  tranNu = tparam(2) + nt;
  dataNu = dparam(2) + nd;
  kappa  = dparam(3) + nd;

  % cached statistics of transformations and data, for each class
  tranchol = reshape(pp(1:end-1,:), dim, dim, numcc);
  datamean = qq(1:dim,:) ./ kappa(ones(dim,1),:);
  datachol = reshape(qq(dim+1:end-1,:), dim, dim, numcc);

  % inverse normalization constants for covariance of each class
  trannorm = (tranNu-dim-1);
  datanorm = (kappa.*(dataNu-dim-1)) ./ (kappa+1);

  % compute variance, mean, weight of each cluster, conditioned on depth ray
  lamZ = zeros(1,numcc);
  muZ  = zeros(1,numcc);
  logW = zeros(1,numcc);
  for cc = 1:numcc
    % compute covariance averaging over transformations
    trancov = tranchol(:,:,cc)'*tranchol(:,:,cc) ./ trannorm(cc);
    datacov = datachol(:,:,cc)'*datachol(:,:,cc) ./ datanorm(cc);
    sumchol = chol(trancov + datacov);
    
    scaledV  = sumchol' \ v;
    scaledMu = sumchol' \ datamean(:,cc);

    lamZ(cc) = 1 / sum(scaledV.^2);
    muZ(cc)  = lamZ(cc) * scaledV'*scaledMu;
    
    delmu    = muZ(cc)*scaledV - scaledMu;
    logW(cc) = -sum(log(diag(sumchol))) - 0.5*sum(delmu.^2);
  end

  % evaluate conditioned Gaussians at discrete set of depths
  % 0.5*log(lamZ) term cancels in calculation of mixture weights
  clik = logW(ones(numzz,1),:);
  clik = clik - 0.5*((zz(:,ones(1,numcc))-muZ(ones(numzz,1),:)).^2 ./ ...
    lamZ(ones(numzz,1),:));

case 2,   % Diagonal Inverse-Wishart (constrained zero mean, assumes dtype=3)
  dim    = tparam(1); % = dparam(1);
  tranNu = tparam(2) + nt;
  dataNu = dparam(2) + nd;
  kappa  = dparam(3) + nd;

  % cached statistics of transformations and data, for each class
  tranchol = pp(1:dim,:);
  datamean = qq(1:dim,:) ./ kappa(ones(dim,1),:);
  datachol = qq(dim+1:end-1,:);

  % inverse normalization constants for covariance of each class
  trannorm = (tranNu-dim-1);
  datanorm = (kappa.*(dataNu-dim-1)) ./ (kappa+1);

  % compute covariance averaging over transformations
  trancov = (tranchol.^2) ./ trannorm(ones(dim,1),:);
  datacov = (datachol.^2) ./ datanorm(ones(dim,1),:);
  sumchol = sqrt(trancov + datacov);

  % compute variance, mean, weight of each cluster, conditioned on depth ray
  scaledV  = v(:,ones(1,numcc)) ./ sumchol;
  scaledMu = datamean ./ sumchol;

  lamZ  = 1 ./ sum(scaledV.^2);
  muZ   = lamZ .* sum(scaledV.*scaledMu);
  delmu = muZ(ones(dim,1),:).*scaledV - scaledMu;
  logW  = -sum(log(sumchol)) - 0.5*sum(delmu.^2);

  % evaluate conditioned Gaussians at discrete set of depths
  % 0.5*log(lamZ) term cancels in calculation of mixture weights
  clik = logW(ones(numzz,1),:);
  clik = clik - 0.5*((zz(:,ones(1,numcc))-muZ(ones(numzz,1),:)).^2 ./ ...
    lamZ(ones(numzz,1),:));
end

