
function [llikcc,refcc] = gibbs_llikTable(numiter, ss, oo, conpart, partnd, ...
  posqq, obsqq, objpp, objnt, stype, sparam, otype, oparam, ttype, tparam)
%gibbs_llikTable    Auxiliary sampler for likelihood of table reassignments

% process input arguments
numdata = size(ss,2);
numpart = length(partnd) - 1;

% temporary variables for auxiliary Gibbs sampler
datakk = zeros(numdata,1);
refcc  = zeros(size(ss,1),1); % TODO: generic transforms
ssref  = ss;

% appearance likelihood (static across later sampling)
clikoo = datastat_indeplike(obsqq, partnd, oo, otype, oparam);

% part probabilities, allowing for conpart=0 (static across later sampling)
probkk = [partnd(1:numpart) conpart];
normkk = sum(probkk);
if normkk > 0
  probkk = probkk/normkk;
else
  probkk = [zeros(1,numpart), 1];
end
probkk = probkk(ones(numdata,1),:);

% perform specified number of resampling steps
for iter = 1:numiter
  % compute likelihood of all features
  if iter > 1   % initialize based on appearance only
    clik = clikoo + datastat_indeplike(posqq, partnd, ssref, stype, sparam);
  else
    clik = clikoo;
  end

  % compute unnormalized posterior probability for each feature
  maxlike = max(clik,[],2);
  clik    = exp(clik-maxlike(:,ones(1,numpart+1))) .* probkk;

  % vectorized sampling of new assignments for all features
  cumprob  = cumsum(clik, 2); 
  randvals = rand(numdata,1) .* cumprob(:,numpart+1);
  datakk   = sum((randvals(:,ones(1,numpart+1)) > cumprob), 2) + 1;

  % sample a new transformation corresponding to these part assignments
  refcc = trandata_partsamp(posqq, partnd, objpp, objnt, ss, datakk, ...
    stype, sparam, ttype, tparam);
  ssref = ss - refcc(:,ones(1,numdata)); % TODO: generic transforms
end

% compute likelihood given transformation, marginalizing part assignments
clik = clikoo + datastat_indeplike(posqq, partnd, ssref, stype, sparam);

maxlike  = max(clik');
normlike = exp(clik' - maxlike(ones(numpart+1,1),:));
llikcc   = sum(log(probkk(1,:)*normlike+eps) + maxlike);

