
function rr = trandata_samp(qq, nd, pp, nt, ss, kk, dtype, dparam, ttype, tparam)
%trandata_partsamp    Sample single transform given data assigned to several parts

switch ttype,
case 1,   % Inverse-Wishart (constrained zero mean, assumes dtype=2)
  dim    = tparam(1); % = dparam(1);
  tranNu = tparam(2) + nt;
  dataNu = dparam(2) + nd;
  kappa  = dparam(3) + nd;
  numcc  = size(qq,2);
  numobs = size(ss,2); 

  % cached statistics of transformations and data
  tranchol = reshape(pp(1:end-1), dim, dim);
  datamean = qq(1:dim,:) ./ kappa(ones(dim,1),:);
  datachol = reshape(qq(dim+1:end-1,:), dim, dim, numcc);

  % inverse normalization constants for covariance of each class
  trannorm = (tranNu-dim-1);
  datanorm = (kappa.*(dataNu-dim-1))./(kappa+1);

  % compute inverse covariance for prior and each part
  trancholinv = tranchol \ eye(dim);
  traninv     = trannorm*trancholinv*trancholinv';

  datainv = zeros(dim,dim,numcc);
  for cc = 1:numcc
    datacholinv     = datachol(:,:,cc) \ eye(dim);
    datainv(:,:,cc) = datanorm(cc)*datacholinv*datacholinv';
  end

  % compute posterior inverse covariance
  postinv     = traninv + sum(datainv(:,:,kk),3);
  postinvchol = chol(postinv);

  % compute posterior mean
  postmean = zeros(dim,1);
  for ii = 1:numobs
    postmean = postmean + datainv(:,:,kk(ii)) * (ss(:,ii) - datamean(:,kk(ii)));
  end
  postmean = postinv \ postmean;

  % single sample from posterior distribution
  rr = postmean + postinvchol \ randn(dim,1);

case 2,   % Diagonal Inverse-Wishart (constrained zero mean, assumes dtype=3)
  dim    = tparam(1); % = dparam(1);
  tranNu = tparam(2) + nt;
  dataNu = dparam(2) + nd;
  kappa  = dparam(3) + nd;
  numcc  = size(qq,2);
  numobs = size(ss,2); 

  % cached statistics of transformations and data
  tranchol = pp(1:dim);
  datamean = qq(1:dim,:) ./ kappa(ones(dim,1),:);
  datachol = qq(dim+1:end-1,:);

  % inverse normalization constants for covariance of each class
  trannorm = (tranNu-dim-1);
  datanorm = (kappa.*(dataNu-dim-1))./(kappa+1);

  % compute inverse covariance for prior and each part
  traninv = trannorm ./ (tranchol.^2);
  datainv = datanorm(ones(dim,1),:) ./ (datachol.^2);

  % compute posterior inverse covariance
  postinv = traninv + sum(datainv(:,kk),2);

  % compute posterior mean
  postmean = sum(datainv(:,kk) .* (ss-datamean(:,kk)), 2) ./ postinv;

  % single sample from posterior distribution
  rr = postmean + randn(dim,1) ./ sqrt(postinv);

end

