
%test_dpmix     Synthetic data test of flat Dirichlet process mixture model

%%%%%%%%%%%%%
%% Parameters
%%%%%%%%%%%%%

% clustering hyperparameters
alpha = 2;

useApp = 0; % appearance features: 0=none, 1=discrete, 2=Gaussian (diagonal)
usePos = 1; % 1 to use Gaussian feature position model, 0 neglects positions

% Gaussian position hyperparameters and visualization
dataprior{1}.type    = 'gaussian';
%dataprior{1}.type    = 'diagonalgaussian';
dataprior{1}.emean   = zeros(2,1);
dataprior{1}.cmean   = 0.1;
dataprior{1}.ecovar  = eye(2);
dataprior{1}.ccovar  = 6;
plotstyle{1}.numbers = 0;
plotstyle{1}.range   = 10*[-1 1 -1 1];

% Discrete appearance hyperparameters and visualization
dataprior{2}.type = 'discrete';
dataprior{2}.numfeature = 30;
dataprior{2}.dirweight  = 0.1;
plotstyle{2} = struct([]);

% Diagonal Gaussian appearance hyperparameters and visualization
appdim = 10;  % dimension of diagonal Gaussian appearance features
dataprior{3}.type   = 'diagonalgaussian';
dataprior{3}.emean  = zeros(appdim,1);
dataprior{3}.cmean  = 0.1;
dataprior{3}.ecovar = ones(appdim,1);
dataprior{3}.ccovar = appdim+5;  % must be larger than appdim+1
plotstyle{3}.meanscale  = [-10 10];

% Test corpus parameters
numdata  = 1000;

% Learning parameters
numgibbs   = 100;
numconiter = 30;

%%%%%%%%%%%%%%
%% Test script
%%%%%%%%%%%%%%

% Select appropriate data hyperparameters
if useApp==1
  dataprior = dataprior(1:2);
  plotstyle = plotstyle(1:2);
elseif useApp==2
  dataprior = dataprior([1 3]);
  plotstyle = plotstyle([1 3]);
else
  dataprior = dataprior(1);
  plotstyle = plotstyle(1);
end
if ~usePos
  dataprior = dataprior(2);
  plotstyle = plotstyle(2);
end

% Create figures to display inference results
for dd = 1:length(plotstyle)
  Htgt(dd) = figure('name', 'Ground truth');
  Hinf(dd) = figure('name', 'Inferred parts');
end

% Construct and visualize synthetic data set
DPtgt = tdpmix(1, alpha, inf, dataprior);
DPtgt = sampledata(DPtgt, [], numdata);
[tgtParam,tgtProb] = gettables(DPtgt,1);
tgtParam = tgtParam{1}; tgtProb = tgtProb{1};

visualize(DPtgt, 1, plotstyle, Htgt);
drawnow;

% Clear part assignments used to generate data
DPinf = clearassign(DPtgt);

% Gibbs sampling learning
DPinf = doGibbs(DPinf, numgibbs, {1, plotstyle, Hinf}, 1, [], numconiter);
[infParam,infProb] = gettables(DPinf,1);
infParam = infParam{1}; infProb = infProb{1};

