
%test_tdpmix    Synthetic data test of Transformed Dirichlet Process

%%%%%%%%%%%%%
%% Parameters
%%%%%%%%%%%%%

% clustering hyperparameters
gamma = 5;
alpha = 1;

useApp = 0; % appearance features: 0=none, 1=discrete, 2=Gaussian (diagonal)

% Gaussian position hyperparameters and visualization
dataprior{1}.type    = 'gaussian';
%dataprior{1}.type    = 'diagonalgaussian';
dataprior{1}.emean   = zeros(2,1);
dataprior{1}.cmean   = 0.1;
dataprior{1}.ecovar  = 2*eye(2);
dataprior{1}.ccovar  = 6;
plotstyle{1}.numbers = 0;
plotstyle{1}.range   = 20*[-1 1 -1 1];

% Discrete appearance hyperparameters and visualization
dataprior{2}.type = 'discrete';
dataprior{2}.numfeature = 30;
dataprior{2}.dirweight  = 0.1;
plotstyle{2} = struct([]);

% Diagonal Gaussian appearance hyperparameters and visualization
appdim = 10;  % dimension of diagonal Gaussian appearance features
dataprior{3}.type   = 'diagonalgaussian';
dataprior{3}.emean  = zeros(appdim,1);
dataprior{3}.cmean  = 0.1;
dataprior{3}.ecovar = ones(appdim,1);
dataprior{3}.ccovar = appdim+5;  % must be larger than appdim+1
plotstyle{3}.meanscale  = [-10 10];

% Transformation hyperparameters
tranprior.type   = 'zeromeangaussian';
%tranprior.type   = 'zeromeandiagonalgaussian';
tranprior.ccovar = 6;
tranprior.ecovar = 5*eye(2);

% Test corpus parameters
numgroup = 5;
numdata  = 300;
maxclass = 3;
maxtable = 5;

% Learning parameters
numgibbs   = 100;
numconiter = 30;

%%%%%%%%%%%%%%
%% Test script
%%%%%%%%%%%%%%

% Select appropriate data hyperparameters
if useApp==1
  dataprior = dataprior(1:2);
  plotstyle = plotstyle(1:2);
elseif useApp==2
  dataprior = dataprior([1 3]);
  plotstyle = plotstyle([1 3]);
else
  dataprior = dataprior(1);
  plotstyle = plotstyle(1);
end

% Create figures to display inference results
for dd = 1:length(plotstyle)
  Htgt(dd) = figure('name', 'Ground truth');
  Hinf(dd) = figure('name', 'Inferred parts');
end

% Construct and visualize synthetic data set
HDPtgt = tdpmix(numgroup, alpha, gamma, dataprior, tranprior);
HDPtgt = sampledata(HDPtgt, [], numdata, maxclass, maxtable);
[tgtParam,tgtProb] = getclasses(HDPtgt);

visualize(HDPtgt, [1:min(5,numgroup)], plotstyle, Htgt);
drawnow;

% Clear part assignments used to generate data
HDPinf = clearassign(HDPtgt);

% Gibbs sampling learning
HDPinf = doGibbs(HDPinf, numgibbs, {[1:min(5,numgroup)], plotstyle, Hinf}, ...
  [1:numgroup], [], numconiter);
[infParam,infProb] = getclasses(HDPinf);

