
function words = getWords(descriptor, Shape, ROItype, dictionary, encodeShape)
% words = getWords(centers, descriptor, locations)
%
% The function can be used in two ways:
% 1) Returns the cluster index for each feature
% words = getWords(centers, descriptor);
%
% 2) Enhances dictionary with aspect ratio and orientation information
% words = getWords(centers, descriptor, locations)

if nargin < 5
    encodeShape = 0;
end

% make sure it is double:
[Dim n] = size(descriptor);
descriptor = double(descriptor);

centers = dictionary.clusters;
[NumberWordsPerROI, Dim, nROI] = size(centers);

% find closest center to each descriptor:
words = zeros(1,n);
for t = 1:nROI
    j = find(ROItype == t);
    [words(j), d] = matchDescriptors(descriptor(:,j), centers(:,:,t)');
    words(j) = words(j) + (t-1)*NumberWordsPerROI;
end
NumberWords = NumberWordsPerROI*nROI; % number of words

% Enhance dictionary with aspect ratio and orientation information
if encodeShape
    for t = 1:nROI
        j = find(ROItype == t);
        [orientation, scale, aspectRatio] = getROIproperties(Shape(:,j));

        % Enhances the dictionary by making the size of the dictionary three
        % times bigger by taking into account the shape of the region.
        % First looks at the aspect ratio. One group is for regions with
        % aspect ratio is < 2. Then for aspectRatio>2 there are two groups
        % according to the orientation of the region (>45 or <45 degrees)
        if t == 3
            wordOrient = abs(cos(orientation)) < sqrt(2)/2;
            ndx = wordOrient;
        else
            % Harris affine and MSER have three kinds of shapes:
            wordOrient = abs(cos(orientation)) < sqrt(2)/2;
            wordScale  = aspectRatio > 2; % in the case of edges ROI only orientation makes sense.
            ndx = wordScale.*(1+wordOrient);
        end
        words(j) = words(j) + ndx'*NumberWords;
    end
end

function [j, d] = matchDescriptors(feat1, feat2)
%
% Computes closest match.
% j is the index to feat2 indicating the closest match to feat1
%

n1 = size(feat1, 2);
n2 = size(feat2, 2);

d = zeros(n1,1);
j = zeros(n1,1);

s1 = sum(feat1(6:end,:).^2);
s2 = sum(feat2(6:end,:).^2);
for n = 1:n1
    %D = distMat(feat1(6:end,n1)', feat2(6:end,:)');
    D = s1(n) + s2 - 2*feat1(6:end,n)'*feat2(6:end,:);
    [d(n) j(n)]= min(D);
end
