
% Visualization of TDP scenes: global object clusters with parts
function scenepartTDP_visglob(TDP, objstyle, globthresh, partthresh, scenerange, Hfig)

if nargin < 3
  globthresh = 1.0;
end
if nargin < 4
  partthresh = 1.0;
end
if nargin < 5
  scenerange = [0 256 0 256];
end

if nargin < 6
  Hfig = figure;
elseif Hfig ~= 0
  clf(Hfig);
end

% plot parameters
Ncol = 4;

% global clusters and corresponding transformation densities
[partParam,partProb,objProb,objTran,objLabel] = getobjects(TDP);
numobj = length(objLabel);  % number of visual objects (global clusters)

% sort global object clusters according to probabilities
[objProb,sortind] = sort(-objProb);
objProb   = -objProb;
partParam = partParam(sortind);
partProb  = partProb(sortind);
objTran   = objTran(:,sortind);
objLabel  = max(objLabel(sortind),0);  % ignore negative labels

% extract mean & covariance of parts for each object
dim = 2;
muPart  = cell(1,numobj);
sigPart = cell(1,numobj);

for cc = 1:numobj
  numpart(cc) = length(partProb{cc});
  muPart{cc}  = partParam{cc}{1}(1:dim,:);

  if size(partParam{cc}{1},1) > 2*dim
    sigPart{cc} = reshape(partParam{cc}{1}(dim+1:end,:),dim,dim,numpart(cc));
  else
    sigPart{cc} = zeros(dim,dim,numpart(cc));
    for kk = 1:numpart(cc)
      sigPart{cc}(:,:,kk) = diag(partParam{cc}{1}(dim+1:end,kk));
    end
  end
end

% extract covariance of transformations for each object
if size(objTran,1) > dim
  sigTran = reshape(objTran,dim,dim,numobj);
else
  sigTran = zeros(dim,dim,numobj);
  for cc = 1:numobj
    sigTran(:,:,cc) = diag(objTran(:,cc));
  end
end

% normalize means for current positions of assigned tables
[tableObj,tableProb,tableTran] = gettables(TDP);
numgroup = length(tableObj);
muTables = zeros(dim,numobj);
objnt    = zeros(1,numobj);

for jj = 1:numgroup
  numtable = length(tableObj{jj});

  for tt = 1:numtable
    cc = tableObj{jj}(tt);
    muTables(:,cc) = muTables(:,cc) + tableTran{jj}(:,tt);
    objnt(cc)      = objnt(cc) + 1;
  end
end
muTables = muTables ./ objnt(ones(dim,1),:);
muTables = muTables(:,sortind);

for cc = 1:numobj
  muPart{cc} = muPart{cc} + muTables(:,cc*ones(1,numpart(cc)));
end

% threshold display of global clusters with small probabilities
cumProb = cumsum(objProb);
numplot = min(find(cumProb > globthresh-eps));
Nrow = ceil(numplot/Ncol);

% plot segmented features for each image
for cc = 1:numplot
  if Hfig == 0
    Hnew = figure;
    Hax  = subplot(1,1,1,'parent',Hnew);
  else
    Hax = subplot(Nrow,Ncol,cc,'parent',Hfig);
    cla(Hax);
  end

  % exclude low-probability parts
  [sortProb,sortind] = sort(-partProb{cc});
  sortProb = -sortProb;
  muSort   = muPart{cc}(:,sortind);
  sigSort  = sigPart{cc}(:,:,sortind);
  cumProb  = cumsum(sortProb);
  partplot = min(find(cumProb > partthresh-eps));

  plotGaussMix(muSort(:,1:partplot), sigSort(:,:,1:partplot), sortProb(1:partplot),...
    objstyle{objLabel(cc)+1}, Hax, {}, {'-k', 'LineWidth', 2}); 
  hold(Hax,'on');
  plotGaussMix(muPart{cc}*partProb{cc}', sigTran(:,:,cc), 1, ...
    'none', Hax, {}, {'--k', 'LineWidth', 2}); 

  axis(Hax,'equal'); axis(Hax,'ij'); axis(Hax,scenerange);
  set(Hax,'box','on'); set(Hax,'xtick',[]);  set(Hax,'ytick',[]);
  title(Hax, sprintf('%.3g',objProb(cc)));
end

