
% HDP/DP (shared/unshared) models for object categories
% Test signicance of differences in area under ROC

close all; clear all;
load sharing_ijcv16;    

% load result file(s): 
datapath = 'TODO';
result{1}{1,1} = strcat(datapath, 'PHD06-03-30/posS6a_test12');
result{1}{1,2} = strcat(datapath, 'PHD06-03-30/posS6b_test30');
result{2}{1,1} = strcat(datapath, 'IJCV05-09-29/posN_test12');
result{2}{1,2} = strcat(datapath, 'IJCV05-09-29/posN_test30');
result{3}{1,1} = strcat(datapath, 'PHD06-03-30/appS6_test30');
result{4}{1,1} = strcat(datapath, 'IJCV05-09-29/appN_test30');

% choose indices of training examples for which plots created
indA = [1:8];   % training indices for which significance statistics computed

% combine variables storing simulation results from multiple simulation runs
numtrial  = length(result);
params    = load(result{1}{1}, 'numobj', 'testindex');
numobj    = params.numobj;
%testindex = params.testindex;

numtrainO = [];
for jj = 1:size(result{1},2)
  params = load(result{1}{1,jj}, 'numtrainO');
  numtrainO = [numtrainO, params.numtrainO];
end

% avoid excessive memory usage when loading results
trainhist = cell(length(numtrainO),numtrial);
for ii = 1:numtrial
  resultind = 1;

  [numseed,numproc] = size(result{ii});

  for jj = 1:numproc
    for ss = 1:numseed
      traintemp = load(result{ii}{ss,jj}, 'trainhist');

      if ss == 1
        numresult = length(traintemp.trainhist);
        totalsamp = zeros(numresult,1);
      else
        if length(traintemp.trainhist) ~= numresult
          error('Different random seeds must use same training cases');
        end
      end

      for kk = 1:numresult
        if isempty(traintemp.trainhist{kk})
          break;
        end

        if isfield(traintemp.trainhist{kk}, 'DPinfB')
          traintemp.trainhist{kk}.DPinfB = {};
        end
        traintemp.trainhist{kk}.DPinfO = {};
      end

      if ss == 1
        trainhist(resultind:resultind+numresult-1,ii) = traintemp.trainhist;
      else
        for kk = 1:numresult
          if (trainhist{resultind+kk-1,ii}.traingroup ~= ...
              traintemp.trainhist{kk}.traingroup)
            error('Training groups for different seeds inconsistent');
          end
          trainhist{resultind+kk-1,ii}.probO = cat(3, ...
            trainhist{resultind+kk-1,ii}.probO, traintemp.trainhist{kk}.probO);
        end
      end

      clear traintemp;
    end

    resultind = resultind + numresult;
  end
end

% determine mean covariance for detection and recognition score statistics
detectStat = cell(numobj,length(numtrainO));
recogStat  = cell(numobj,length(numtrainO));

% determine area under ROC for different numbers of training examples
for k = 1:length(indA)
  kk = indA(k);

  % extract test groups, and true category labels
  testgroup = trainhist{kk}.testgroup;
  truecat   = imagedata.classes(testgroup);

  % detection and recognition performance
  for ii = 1:numtrial
    objllik  = zeros(numobj, length(testgroup), 0);
    backllik = zeros(1, length(testgroup), 0);
    for jj = 1:size(trainhist{kk,ii}.probO,3);
      probO = trainhist{kk,ii}.probO(:,:,jj);
      if min(min(probO)) < 0
        objllik  = cat(3, objllik, probO(1:numobj,:));
        backllik = cat(3, backllik, probO(numobj+1,:));
      end
    end

    % use most likely sample for each image
    objllik  = max(objllik, [], 3);
    backllik = max(backllik, [], 3);

    % find log-likelihood scores for detection comparisons
    backind = find(truecat==0);
    for oo = 1:numobj
      objind = find(truecat==oo);
      detectStat{oo,kk}.scoreObj(:,ii)  = objllik(oo,objind)' - backllik(objind)';
      detectStat{oo,kk}.scoreBack(:,ii) = objllik(oo,backind)' - backllik(backind)';
    end

    % find posterior probabilities for recognition comparisons
    objllik = objllik/100;
    postprob = exp(objllik - repmat(max(objllik,[],1),numobj,1));
    postprob = postprob ./ repmat(sum(postprob,1),numobj,1);
    for oo = 1:numobj
      objind   = find(truecat==oo);
      otherind = setdiff(find(truecat > 0),objind);
      recogStat{oo,kk}.scoreObj(:,ii)   = postprob(oo,objind);
      recogStat{oo,kk}.scoreOther(:,ii) = postprob(oo,otherind);
    end
  end

  % compute mean and covariance statistics
  for oo = 1:numobj
    [mu,sigma] = mannWhitney(detectStat{oo,kk}.scoreObj, ...
      detectStat{oo,kk}.scoreBack);
    detectStat{oo,kk}.mu = mu;
    detectStat{oo,kk}.sigma = sigma;

    [mu,sigma] = mannWhitney(recogStat{oo,kk}.scoreObj, ...
      recogStat{oo,kk}.scoreOther);
    recogStat{oo,kk}.mu = mu;
    recogStat{oo,kk}.sigma = sigma;
  end
end

return

save shareStats detectStat recogStat numobj;

%%
%% Detection results
%%
%L  = [1 -1 0 0]; % shared vs. unshared position
%L  = [0 0 1 -1]; % shared vs. unshared appearance
%L  = [1 0 -1 0]; % position vs. appearance shared
L  = [0 1 0 -1]; % position vs. appearance unshared
kk = 6; % indexes training set sizes

muDiff  = zeros(numobj,1);
stdDiff = zeros(numobj,1);
pDiff   = zeros(numobj,1);
for oo = 1:numobj
  muDiff(oo)  = L*detectStat{oo,kk}.mu;
  stdDiff(oo) = sqrt(L*detectStat{oo,kk}.sigma*L');
  pDiff(oo)   = 2*normcdf(muDiff(oo),0,stdDiff(oo)) - 1;
end
pDiff
disp(sum(pDiff > 0.95))
disp(sum(pDiff <-0.95))

%%
%% Recognition results
%%
%L  = [1 -1 0 0]; % shared vs. unshared position
%L  = [0 0 1 -1]; % shared vs. unshared appearance
%L  = [1 0 -1 0]; % position vs. appearance shared
L  = [0 1 0 -1]; % position vs. appearance unshared
kk = 2; % indexes training set sizes

muDiff  = zeros(numobj,1);
stdDiff = zeros(numobj,1);
pDiff   = zeros(numobj,1);
for oo = 1:numobj
  muDiff(oo)  = L*recogStat{oo,kk}.mu;
  stdDiff(oo) = sqrt(L*recogStat{oo,kk}.sigma*L');
  pDiff(oo)   = 2*normcdf(muDiff(oo),0,stdDiff(oo)) - 1;
end
pDiff
disp(sum(pDiff > 0.95))
disp(sum(pDiff <-0.95))

