
% Alternative visualization of parts learned by HDP model

clear all; close all;
load sharing_ijcv16;    

datapath = 'TODO';
load(strcat(datapath,'posS6b_test30.mat'), 'trainhist', 'usePos');
DP = trainhist{end}.DPinfO{end};
imagefolder = '../databaseIJCV05/imagesSharingIJCV/';

% extract classes/tables from learned model
[classParam,classProb,classTran] = getclasses(DP);
[tableClass,tableProb,tableTran] = gettables(DP);
numobj   = length(tableProb);
numclass = length(classProb);

% determine which features will be plotted along with each part
% (>0 for threshold on posterior probability, =0 for only most likely features)
probthresh = 0.2;

% plot parameters
%classind = [1:10];
classind  = [43 63 108 113 114 125 133]; % indices of parts to visualize 

% set plot colors (different color for background clutter parts)
%featcolor = repmat([1 1 0], length(classind), 1);
featcolor = [repmat([1 1 0], 6, 1); repmat([0 1 1], 1, 1)];

% choose two visually distinctive images from each category
numimage = 2;
imageind = repmat([1 2], 16, 1);
imageind(3,:)  = [1 6];
imageind(5,:)  = [1 3];
imageind(6,:)  = [1 7];
imageind(7,:)  = [4 5];
imageind(8,:)  = [1 4];
imageind(10,:) = [1 6];
imageind(12,:) = [1 6];
imageind(16,:) = [1 4];

% load images used for later display, and compute part assignments
objimage    = cell(numobj,numimage);
objassign   = cell(numobj,numimage);
objposition = cell(numobj,numimage);

fprintf(1,'Loading images for all categories: ');
for oo = 1:numobj
  objind = find(imagedata.classes == oo);
  for ii = 1:numimage
    objimage{oo,ii}    = imread(fullfile(imagefolder, ...
      imagedata.files{objind(imageind(oo,ii))}));
    objposition{oo,ii} = imagedata.position{objind(imageind(oo,ii))};

    testdata{1} = objposition{oo,ii}(1:2,:) + 1;
    testdata{2} = imagedata.appearance{objind(imageind(oo,ii))};
    objassign{oo,ii} = estassign(DP, oo, testdata);
  end
  fprintf(1,'.');
end
fprintf(1,'\n');

% sort parts by H(obj|part)
objProb = zeros(numobj,numclass);
for oo = 1:numobj
  for tt = 1:length(tableProb{oo})
    cc = tableClass{oo}(tt);
    objProb(oo,cc) = objProb(oo,cc) + tableProb{oo}(tt);
  end
end
objProb = (objProb+eps) ./ repmat(sum(objProb+eps,2),1,numclass);

entPart = objProb ./ repmat(sum(objProb,1),numobj,1);
entPart = -sum(entPart.*log(entPart),1);
[entSort,entIndex] = sort(entPart);

% plot position distribution for each part
dim = 2;
classMean  = classParam{1}(1:dim,:);
classCovar = reshape(classParam{1}(dim+1:end,:),dim,dim,numclass);

for jj = classind
  Hfig = figure('name', 'Part position');
  Hax  = subplot(1,1,1,'Parent',Hfig);
  plotGaussMix(classMean(:,entIndex(jj)), classCovar(:,:,entIndex(jj)), ...
    1, 'b', Hax, {'o','MarkerSize',10}, {'-k','LineWidth',3});
  axis(Hax,'square'); axis(Hax,'ij');
  axis(Hax,[0 128 0 128]);
  set(Hax,'box','on'); set(Hax,'xtick',[]);  set(Hax,'ytick',[]);
end

% plot regions associated with each chosen part in turn
Dy = 1/numobj;
Dx = Dy;

for j = 1:length(classind)
  jj = classind(j);
  Hfig = figure('name', 'Part appearance');

  % for each object category, and each image in that category
  for oo = 1:numobj
    for ii = 1:numimage
      if probthresh > 0
        % TDP returns table assignments, HDP class assignments
        if usePos > 1
          error('TDP threshold plotting not implemented');
        else
          plotind = find(objassign{oo,ii}(:,entIndex(jj)) >= probthresh);
        end
      else
        % find all features most likely to be generated by this part
        [partval,partind] = max(objassign{oo,ii}, [], 2);

        % TDP returns table assignments, HDP class assignments
        if usePos > 1
          plotind = find(tableClass{oo}(partind) == entIndex(jj));
        else
          plotind = find(partind == entIndex(jj));
        end
      end

      % plot those features
      axes('position', [(ii-1)*Dx-0.015*(ii-1) (numobj-oo)*Dy Dx Dy]);
      displayROI(objimage{oo,ii}, objposition{oo,ii}(:,plotind), [], featcolor(j,:));
      title(''); colormap(gray(256));
    end
  end
end

