
% Visualization of ICCV05 sharing experiment: learned parts

% load raw data and result file, and extract classes from desired model
load sharing_ijcv16;    
load pos2b_part30 OPinfS;
OP = OPinfS;

imagefolder = '../database/imagesSharingIJCV/';

[classParam,classProb] = getclasses(OP);
[numobj,numclass] = size(classProb);

% plot parameters
numimage  = 6;        % number of images of each category 
classind  = [1:32];   % indices of parts to visualize 
featcolor = [1 1 0];  % RGB color of feature ellipses

% load images used for later display, and compute part assignments
objimage    = cell(numobj,numimage);
objassign   = cell(numobj,numimage);
objposition = cell(numobj,numimage);

fprintf(1,'Loading images for all categories: ');
for oo = 1:numobj
  objind = find(imagedata.classes == oo);
  for ii = 1:numimage
    objimage{oo,ii}    = imread(fullfile(imagefolder,imagedata.files{objind(ii)}));
    objassign{oo,ii}   = estassign(OP, objind(ii), oo);
    objposition{oo,ii} = imagedata.position{objind(ii)};
  end
  fprintf(1,'.');
end
fprintf(1,'\n');

% sort parts by H(obj|part)
entPart = classProb ./ repmat(sum(classProb,1),numobj,1);
entPart = -sum(entPart.*log(entPart),1);
[entSort,entIndex] = sort(entPart);

% plot position distribution for each part
dim = 2;
classMean  = classParam{1}(1:dim,:);
classCovar = reshape(classParam{1}(dim+1:end,:),dim,dim,numclass);

HfigG = figure(numclass+1);
set(HfigG, 'name', 'Part Position Densities');
for jj = 1:numclass
  Hax = subplot(ceil(numclass/8),8,jj,'Parent',HfigG);
  plotGaussMix(classMean(:,entIndex(jj)), classCovar(:,:,entIndex(jj)), 1, [], Hax);
  axis(Hax,'square'); axis(Hax,'ij');
  axis(Hax,[0 128 0 128]);
  set(Hax,'box','on'); set(Hax,'xtick',[]);  set(Hax,'ytick',[]);
  title(Hax, sprintf('%d',jj));
end
input(sprintf('Part position densities...',jj));

% plot regions associated with each chosen part in turn
Dx = 1/numimage; 
Dy = 1/numobj;

for jj = classind
  HfigP = figure(jj);
  set(HfigP, 'name', sprintf('Part %d (ordered by entropy)',jj));

  % for each object category, and each image in that category
  for oo = 1:numobj
    for ii = 1:numimage
      % find all features most likely to be generated by this part
      [partval,partind] = max(objassign{oo,ii}, [], 2);
      plotind = find(partind == entIndex(jj));

      % plot those features
      axes('position', [(ii-1)*Dx (numobj-oo)*Dy Dx Dy]);
      displayROI(objimage{oo,ii}, objposition{oo,ii}(:,plotind), [], featcolor);
      title(''); colormap(gray(256));
    end
  end
  
  input(sprintf('Part %d...',jj));
end

% compute distances between distributions over parts
D = zeros(numobj);
for ii = 1:numobj
  for jj = 1:numobj
    D(ii,jj) = sum(classProb(ii,:).*log(classProb(ii,:)./classProb(jj,:))) + ...
               sum(classProb(jj,:).*log(classProb(jj,:)./classProb(ii,:)));
  end
end

% dendrogram
Z = linkage(squareform(D), 'single');
HfigD = figure(numclass+2);
set(HfigD, 'name', 'Category Distance Dendrogram');
Hax = subplot(1,1,1,'Parent',HfigD);
[Hline,ind,perm] = dendrogram(Z,'colorthreshold','default','orientation','right');
set(Hax, 'Yticklabel', imagedata.classnames(perm));
set(Hax, 'FontSize', 8);

% MDS embedding
coord = mdscale(D,2);
coord = coord - min(min(coord));
coord = 0.85*coord/max(max(coord));

HfigM = figure(numclass+3);
set(HfigM, 'name', 'Category Distance Embedding');
for oo = 1:numobj
  axes('position', [coord(oo,1) coord(oo,2) .15 .15])
  image(objimage{oo,1});
  title(imagedata.classnames{oo});
  hold on;
  plot(64,64,'or','markerfacecolor','r');
  axis('off'); axis('equal'); axis('tight');
  colormap(gray(256));
end

